#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of flask-debugtb-elasticsearch.
# https://github.com/heynemann/flask-debugtb-elasticsearch

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Bernardo Heynemann <heynemann@gmail.com>


import json
import jinja2
from flask_debugtoolbar.panels import DebugPanel
from flask_debugtoolbar_elasticSearch.utils import ThreadCollector

from elasticsearch.connection.base import Connection


collector = ThreadCollector()


old_log_request_success = Connection.log_request_success

_ = lambda x: x


def patched_log_request_success(self, method, full_url, path, body, status_code, response, duration):
    collector.collect(ElasticQueryInfo(method, full_url, path, body, status_code, response, duration))
    old_log_request_success(self, method, full_url, path, body, status_code, response, duration)

Connection.log_request_success = patched_log_request_success


def _pretty_json(data):
    # pretty JSON in tracer curl logs
    try:
        return json.dumps(json.loads(data), sort_keys=True, indent=2, separators=(',', ': ')).replace("'", r'\u0027')
    except (ValueError, TypeError):
        # non-json data or a bulk request
        return data


class ElasticQueryInfo():
    def __init__(self, method, full_url, path, body, status_code, response, duration):
        self.method = method
        self.full_url = full_url
        self.path = path
        self.body = _pretty_json(body)
        self.status_code = status_code
        self.response = _pretty_json(response)
        self.duration = round(duration * 1000, 2)


class ElasticsearchDebugPanel(DebugPanel):
    name = 'Elastic'
    has_content = True
    records = []

    def __init__(self, *args, **kwargs):
        super(ElasticsearchDebugPanel, self).__init__(*args, **kwargs)
        self.jinja_env.loader = jinja2.ChoiceLoader([
            self.jinja_env.loader,
            jinja2.PrefixLoader({
                'debug_tb_elastic': jinja2.PackageLoader(__name__, 'templates')
            })
        ])

    def nav_title(self):
        return _('Elastic Queries')

    def nav_subtitle(self):
        return "{} queries {}ms".format(self.nb_queries, self.total_time)

    def title(self):
        return _('Elastic')

    def process_request(self, request):
        collector.clear_collection()

    def url(self):
        return ''

    def process_response(self, request, response):
        records = collector.get_collection()
        self.total_time = 0

        for record in records:
            self.total_time += record.duration

        self.nb_queries = len(records)

        collector.clear_collection()
        self.records = records

    def content(self):
        context = self.context.copy()
        context.update({
            'records': self.records,
            'debug': True,
        })

        return self.render('debug_tb_elastic/elasticsearch.html', context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of flask-debugtb-elasticsearch.
# https://github.com/heynemann/flask-debugtb-elasticsearch

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Bernardo Heynemann <heynemann@gmail.com>

from unittest import TestCase as PythonTestCase


class TestCase(PythonTestCase):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of flask-debugtb-elasticsearch.
# https://github.com/heynemann/flask-debugtb-elasticsearch

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Bernardo Heynemann <heynemann@gmail.com>


import threading


class ThreadCollector(object):
    def __init__(self):
        if threading is None:
            raise NotImplementedError(
                "threading module is not available, "
                "this panel cannot be used without it")
        self.collections = {}  # a dictionary that maps threads to collections

    def get_collection(self, thread=None):
        """
        Returns a list of collected items for the provided thread, of if none
        is provided, returns a list for the current thread.
        """
        if thread is None:
            thread = threading.currentThread()
        if thread not in self.collections:
            self.collections[thread] = []
        return self.collections[thread]

    def clear_collection(self, thread=None):
        if thread is None:
            thread = threading.currentThread()
        if thread in self.collections:
            del self.collections[thread]

    def collect(self, item, thread=None):
        self.get_collection(thread).append(item)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of flask-debugtb-elasticsearch.
# https://github.com/heynemann/flask-debugtb-elasticsearch

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Bernardo Heynemann <heynemann@gmail.com>

from setuptools import setup, find_packages
from flask_debugtb_elasticsearch import __version__

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'ipdb',
    'coveralls',
    'sphinx',
]

setup(
    name='flask-debugtb-elasticsearch',
    version=__version__,
    description='Flask debug toolbar panel for elasticsearch queries.',
    long_description='''
Flask debug toolbar panel for elasticsearch queries.
''',
    keywords='flask elasticsearch debug toolbar',
    author='Bernardo Heynemann',
    author_email='heynemann@gmail.com',
    url='https://github.com/heynemann/flask-debugtb-elasticsearch',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Flask>=0.8',
        'elasticsearch',
        'Flask-DebugToolbar',
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            # 'flask-debugtb-elasticsearch=flask_debugtb_elasticsearch.cli:main',
        ],
    },
)
